@@ -113,14 +113,15 @@ class DataProvidersTest(parameterized.TestCase):
         batch_size=batch_size,
         params=params,
         drop_remainder=False,
-        inference=inference)
+        inference=inference,
+        example_label_tuple=True)
     total = 0
-    for subreads, label in dataset.as_numpy_iterator():
+    for rows, label in dataset.as_numpy_iterator():
       # Last batch may contain fewer examples.
       if not inference:
-        self.assertLen(subreads, len(label))
-      self.assertLessEqual(len(subreads), batch_size)
-      total += len(subreads)
+        self.assertLen(rows, len(label))
+      self.assertLessEqual(len(rows), batch_size)
+      total += len(rows)
     self.assertEqual(total, num_epochs * dataset_size)
 
   @parameterized.named_parameters(
@@ -175,18 +176,25 @@ class DataProvidersTest(parameterized.TestCase):
         batch_size=batch_size,
         params=params,
         drop_remainder=False,
-        inference=inference,
-        keep_metadata=True)
+        inference=inference)
     total = 0
-    for subreads, label, num_passes in dataset.as_numpy_iterator():
+    for tf_example in dataset.as_numpy_iterator():
+      rows = tf_example['rows']
+      label = tf_example['label']
+      num_passes = tf_example['num_passes']
+      window_pos = tf_example['window_pos']
+      name = tf_example['name']
       # Last batch may contain fewer examples.
+      actual_batch_size = len(rows)
       if not inference:
-        self.assertLen(subreads, len(label))
-      self.assertLessEqual(len(subreads), batch_size)
+        self.assertLen(label, actual_batch_size)
+      self.assertLen(window_pos, actual_batch_size)
+      self.assertLen(name, actual_batch_size)
+      self.assertLessEqual(actual_batch_size, batch_size)
       # Sanity check the values in the num_passes array.
       self.assertTrue(tf.reduce_all(num_passes <= 20))
       self.assertTrue(tf.reduce_all(num_passes > 0))
-      total += len(subreads)
+      total += actual_batch_size
     self.assertEqual(total, num_epochs * dataset_size)
 
   @parameterized.named_parameters(
@@ -215,16 +223,17 @@ class DataProvidersTest(parameterized.TestCase):
         params=params,
         inference=inference)
     check_not_empty = False
-    for subreads, _ in dataset.as_numpy_iterator():
+    for data in dataset.as_numpy_iterator():
+      rows = data['rows']
       check_not_empty = True
       base_indices, pw_indices, ip_indices, strand_indices, ccs_indices, sn_indices = data_providers.get_indices(
           params.max_passes)
-      base_rows = subreads[:, slice(*base_indices), :, :]
-      pw_rows = subreads[:, slice(*pw_indices), :, :]
-      ip_rows = subreads[:, slice(*ip_indices), :, :]
-      strand_rows = subreads[:, slice(*strand_indices), :, :]
-      ccs_rows = subreads[:, slice(*ccs_indices), :, :]
-      sn_rows = subreads[:, slice(*sn_indices), :, :]
+      base_rows = rows[:, slice(*base_indices), :, :]
+      pw_rows = rows[:, slice(*pw_indices), :, :]
+      ip_rows = rows[:, slice(*ip_indices), :, :]
+      strand_rows = rows[:, slice(*strand_indices), :, :]
+      ccs_rows = rows[:, slice(*ccs_indices), :, :]
+      sn_rows = rows[:, slice(*sn_indices), :, :]
       self.assertNotEmpty(base_rows)
       self.assertNotEmpty(pw_rows)
       self.assertNotEmpty(ip_rows)
@@ -284,6 +293,40 @@ class DataProvidersTest(parameterized.TestCase):
     limit_dataset_size = sum(1 for record in dataset)
     self.assertEqual(min(limit, full_dataset_size), limit_dataset_size)
 
+  @parameterized.named_parameters(
+      dict(
+          testcase_name='no_qfilter',
+          max_phred_qual=0,
+          expected_n=1301,
+      ),
+      dict(
+          testcase_name='max_phred_qual_30',
+          max_phred_qual=30,
+          expected_n=534,
+      ),
+      dict(
+          testcase_name='max_phred_qual_40',
+          max_phred_qual=40,
+          expected_n=668,
+      ),
+  )
+  def test_dataset_with_max_phred_quality(self, max_phred_qual, expected_n):
+    """Checks that batches are of expected size and all examples yielded."""
+    file_pattern, _ = get_test_dataset(inference=False)
+    params = model_configs.get_config('transformer_learn_values+test')
+    model_utils.modify_params(params)
+    params.max_phred_qual = max_phred_qual
+    # Fetch the complete dataset.
+    full_dataset = data_providers.get_dataset(
+        file_pattern=file_pattern,
+        num_epochs=1,
+        batch_size=1,
+        params=params,
+        inference=False,
+    )
+    full_dataset_size = sum(1 for record in full_dataset)
+    self.assertEqual(full_dataset_size, expected_n)
+
   def test_remove_internal_gaps_and_shift(self):
     label, expected = ('   GGGCGAG   ACATA   ACATA ATA ATA      ',
                        'GGGCGAGACATAACATAATAATA                 ')
