@@ -27,6 +27,7 @@
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 """Tests for losses_and_metrics."""
 
+from typing import Sequence, Tuple
 from absl.testing import absltest
 from absl.testing import parameterized
 import numpy as np
@@ -434,5 +435,187 @@ class AlignmentLossTest(parameterized.TestCase):
     self.assertAlmostEqual(float(loss), expected_loss, places=2)
 
 
+class AlignmentMetricTest(parameterized.TestCase):
+
+  @parameterized.named_parameters(
+      dict(
+          testcase_name='Identical sequences, no pad',
+          sequences=(['TTAGGC', 'AGCTGG'], ['TTAGGC', 'AGCTGG']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(1.0, 1.0)),
+      dict(
+          testcase_name='Two errors, no pad',
+          sequences=(['TTAGGC', 'AGCTGG'], ['AAAGGC', 'TGCTGC']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.667, 0.667)),
+      dict(
+          testcase_name='Correct insertions only, no pad.',
+          sequences=(['TTAGGC', 'AGCTGG'], ['T TA G G C', 'AGC    TGG']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(1.0, 1.0)),
+      dict(
+          testcase_name='One deletion, with pad.',
+          sequences=(['TTAGGC', 'AGCTGG'], ['TTAGG ', 'GCTGG ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.833, 0.833)),
+      dict(
+          testcase_name='One erroneous insertion, no pad.',
+          sequences=(['TTAGGC', 'ATCGAC',
+                      'AGCTGG'], ['TTAGGCA', 'ATCCGAC', 'CAGCTGG']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.857, 0.857, 0.857)),
+      dict(
+          testcase_name='One deletion, shorter, with pad.',
+          sequences=(['ATCG ', 'ATCG '], ['TCG  ', 'TCG  ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.75, 0.75)),
+      dict(
+          testcase_name='Empty predictions.',
+          sequences=(['ATCG ', 'ATCG '], ['     ', '     ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.0, 0.0)),
+      dict(
+          testcase_name='Empty ground-truth.',
+          sequences=(['     ', '     '], ['ATCG ', 'ATCG ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.0, 0.0)),
+      dict(
+          testcase_name='Empty predictions, ground-truth length one.',
+          sequences=(['A    ', 'T    '], ['     ', '     ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.0, 0.0)),
+      dict(
+          testcase_name='Empty ground-truth, predictions length one.',
+          sequences=(['     ', '     '], ['A    ', 'T    ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(0.0, 0.0)),
+      dict(
+          testcase_name='Both empty.',
+          sequences=(['     ', '     '], ['     ', '     ']),
+          matching_score=2.0,
+          mismatch_penalty=5.0,
+          gap_open_penalty=5.0,
+          gap_extend_penalty=4.0,
+          expected_pid=(1.0, 1.0)),  # Expected PID defined as special case.
+  )
+  def test_alignment_metric(self, sequences: Tuple[Sequence[str],
+                                                   Sequence[str]],
+                            matching_score: float, mismatch_penalty: float,
+                            gap_open_penalty: float, gap_extend_penalty: float,
+                            expected_pid: Tuple[float]):
+    y_true, y_pred_scores = test_utils.convert_seqs(sequences)
+    alignment_metric_obj = losses_and_metrics.AlignmentMetric(
+        matching_score=matching_score,
+        mismatch_penalty=mismatch_penalty,
+        gap_open_penalty=gap_open_penalty,
+        gap_extend_penalty=gap_extend_penalty)
+    pid = alignment_metric_obj.alignment(y_true, y_pred_scores)[2]['pid']
+    for i, _ in enumerate(sequences):
+      self.assertAlmostEqual(float(pid[i]), expected_pid[i], places=2)
+
+
+def softmax(x):
+  """Compute softmax values for logits in x."""
+  return np.exp(x) / np.sum(np.exp(x), axis=0)
+
+
+def distill_loss_per_pos_fn_np(teacher_logits, student_logits, temperature):
+  """Computes per position distillation loss."""
+  teacher_probs = softmax(teacher_logits / temperature)
+  student_probs = softmax(student_logits / temperature)
+  # Compute the KL divergence.
+  kl = np.sum(teacher_probs * np.log(teacher_probs / student_probs), axis=-1)
+  return kl
+
+
+class DistillationLossTest(parameterized.TestCase):
+
+  @parameterized.named_parameters(
+      dict(
+          testcase_name='Temperature=1',
+          batch_size=2,
+          window_length=10,
+          temperature=1,
+          are_logits_equal=False,
+      ),
+      dict(
+          testcase_name='Temperature=10',
+          batch_size=2,
+          window_length=10,
+          temperature=10,
+          are_logits_equal=False),
+      dict(
+          testcase_name='Student and teacher have the same logits.',
+          batch_size=2,
+          window_length=10,
+          temperature=1,
+          are_logits_equal=True),
+  )
+  def test_distillation_loss_fn(self, batch_size, window_length, temperature,
+                                are_logits_equal):
+    """Checks that pointwise Distillation values agree with numpy."""
+    # Generate random data.
+    seed_teacher = 0
+    if are_logits_equal:
+      seed_student = seed_teacher
+    else:
+      seed_student = seed_teacher + 1
+    vocab_size = len(dc_constants.VOCAB)
+    np.random.seed(seed_teacher)
+    teacher_logits = np.random.normal(
+        size=(batch_size, window_length, vocab_size))
+    np.random.seed(seed_student)
+    student_logits = np.random.normal(
+        size=(batch_size, window_length, vocab_size))
+
+    distill_loss_fn = losses_and_metrics.DistillationLoss(
+        temperature=temperature, reduction=tf.keras.losses.Reduction.NONE)
+    expected_loss = distill_loss_fn(
+        tf.constant(teacher_logits, dtype=tf.float32),
+        tf.constant(student_logits, dtype=tf.float32)).numpy()
+
+    # Compute distillation loss in numpy pointwise.
+    for example_ind in range(batch_size):
+      distill_loss = 0
+      for pos_ind in range(window_length):
+        kl_ij = distill_loss_per_pos_fn_np(
+            teacher_logits[example_ind, pos_ind, :],
+            student_logits[example_ind, pos_ind, :], temperature)
+        distill_loss = distill_loss + kl_ij
+      # Get the distillation loss over the whole window.
+      distill_loss = distill_loss / window_length
+      self.assertAlmostEqual(distill_loss, expected_loss[example_ind], places=6)
+
+
 if __name__ == '__main__':
   absltest.main()
