@@ -65,47 +65,11 @@ def _set_base_fc_hparams(params):
   params.buffer_size = 1000
 
 
-def _set_base_conv_net_hparams(params, conv_model):
-  """Updates given params with base values for ConvNet models.
-
-  Args:
-    params: configuration dictionary.
-    conv_model: Defines the convolutional network architecture.
-  """
-
-  # Model
-  params.model_name = 'conv_net'
-  params.input_format = 'stack[base,pw,ip,sn]'
-  params.conv_model = conv_model
-
-  # Data
-  params.num_channels = 5
-  params.use_bases = True
-  params.use_pw = True
-  params.use_ip = True
-  params.use_ccs = False
-  params.use_sn = False
-  params.use_strand = False
-  params.per_base_hidden_size = 1
-  params.pw_hidden_size = 1
-  params.ip_hidden_size = 1
-  params.sn_hidden_size = 1
-  params.strand_hidden_size = 1
-
-  # Training
-  params.l2 = 0.0
-  params.batch_size = 256
-  params.num_epochs = 15
-  params.learning_rate = 0.004
-  params.buffer_size = 10000
-
-
 def _set_base_transformer_hparams(params):
   """Updates given config with base values for the Transformer model."""
   # Architecture
   params.model_name = 'transformer'
   params.add_pos_encoding = True
-  params.use_relative_pos_enc = True
   # Num heads should be divisible by hidden size. This value should be tuned for
   # the production setting. TODO: update this parameter after
   # tuning.
@@ -137,6 +101,8 @@ def _set_base_transformer_hparams(params):
 
 def _set_transformer_learned_embeddings_hparams(params):
   """Updates given config with values for the learned embeddings transformer."""
+  # TODO: As we migrate off the legacy code, we might need to
+  # adjust the params below. For now just making a copy of the previous params.
   _set_base_transformer_hparams(params)
   params.model_name = 'transformer_learn_values'
   params.PW_MAX = dc_constants.PW_MAX
@@ -148,6 +114,40 @@ def _set_transformer_learned_embeddings_hparams(params):
   params.ip_hidden_size = 8
   params.strand_hidden_size = 2
   params.sn_hidden_size = 8
+  params.condense_transformer_input = True
+  params.transformer_input_size = 280
+
+
+def _set_transformer_learned_embeddings_distill_hparams(params):
+  """Updates given config with values for the distilled transformer."""
+  _set_transformer_learned_embeddings_hparams(params)
+  params.model_name = 'transformer_learn_values_distill'
+
+  # Student architecture parameters.
+  params.num_hidden_layers = 4
+  params.filter_size = 2048
+
+  # Whether to initialize encoder layers using the teacher model.
+  params.init_encoder_stack = True
+  # Whether to initialize non-encoder layers using the teacher model.
+  params.init_nonencoder_layers = True
+  # The order of indices in params.teacher_encoder_layers and
+  # params.student_encoder_layers determines the layer to layer initialization.
+  params.teacher_encoder_layers = [0, 1, 2, 4]
+  params.student_encoder_layers = [0, 1, 2, 3]
+  if params.init_encoder_stack:
+    assert len(params.teacher_encoder_layers) == len(
+        params.student_encoder_layers)
+    assert len(params.student_encoder_layers) == params.num_hidden_layers
+    assert max(params.student_encoder_layers) < params.num_hidden_layers
+
+  # Distillation loss parameters.
+  # Weight corresponding to the distillation loss.
+  params.distill_alpha = 500
+  # Weight corresponding to the student loss.
+  params.student_alpha = 1
+  # Temperature for softening probability distributions.
+  params.temperature = 1
 
 
 
@@ -187,11 +187,10 @@ def get_config(config_name: str) -> ml_collections.ConfigDict:
 
   Valid config names must consist of two parts: {model_name}+{dataset_name}. The
   "+" must be present as a separator between the two parts. For example,
-  transformer_learn_bases+ccs is a valid name.
+  transformer_learn_values+ccs is a valid name.
 
   Valid model names include:
     * fc
-    * conv_net
     * transformer
     * transformer_learn_values
 
@@ -222,6 +221,9 @@ def get_config(config_name: str) -> ml_collections.ConfigDict:
   params.del_cost = 10.0
   params.loss_reg = 0.1
 
+  # Avg CCS Quality Filter; When set to 0 no examples will be filtered.
+  params.max_phred_qual = 0
+
 
   # Scaling factor to multiply the batch_size when using TPUs since they have
   # more memory than GPUs.
@@ -233,13 +235,12 @@ def get_config(config_name: str) -> ml_collections.ConfigDict:
   params.limit = -1
   if model_config_name == 'fc':
     _set_base_fc_hparams(params)
-  elif model_config_name.startswith('conv_net'):
-    _, conv_sub_model = model_config_name.split('-')
-    _set_base_conv_net_hparams(params, conv_sub_model)
   elif model_config_name == 'transformer':
     _set_base_transformer_hparams(params)
   elif model_config_name == 'transformer_learn_values':
     _set_transformer_learned_embeddings_hparams(params)
+  elif model_config_name == 'transformer_learn_values_distill':
+    _set_transformer_learned_embeddings_distill_hparams(params)
   else:
     raise ValueError('Unknown model_config_name: %s' % model_config_name)
 
@@ -247,6 +248,8 @@ def get_config(config_name: str) -> ml_collections.ConfigDict:
     _set_human_aligned_to_poa_data_hparams(params)
   elif dataset_config_name == 'ccs':
     _set_human_aligned_to_ccs_data_hparams(params)
+  elif dataset_config_name == 'ccs_test':
+    _set_human_test_set_ccs_hparams(params)
   elif dataset_config_name == 'ecoli':
     _set_ecoli_data_hparams(params)
   elif dataset_config_name == 'test':
