@@ -76,36 +76,5 @@ class ModelTrainTest(parameterized.TestCase):
     self.assertLen(best_checkpoint, 1)
 
 
-class GetStepCountsTest(parameterized.TestCase):
-
-  @parameterized.named_parameters(
-      dict(
-          testcase_name='simple',
-          n_examples_train=1000,
-          n_examples_eval=100,
-          batch_size=10,
-          limit=-1,
-          expected_step_counts=(100, 10)),
-      dict(
-          testcase_name='with_limit',
-          n_examples_train=1000,
-          n_examples_eval=100,
-          batch_size=10,
-          limit=100,
-          expected_step_counts=(10, 10)),
-  )
-  def test_get_step_counts(self, n_examples_train, n_examples_eval, batch_size,
-                           limit, expected_step_counts):
-    params = model_configs.get_config('fc+test')
-    with params.unlocked():
-      params.n_examples_train = n_examples_train
-      params.n_examples_eval = n_examples_eval
-      params.limit = limit
-      params.batch_size = batch_size
-
-    self.assertEqual(
-        model_train_custom_loop.get_step_counts(params), expected_step_counts)
-
-
 if __name__ == '__main__':
   absltest.main()
