@@ -35,6 +35,7 @@ from absl.testing import parameterized
 
 import tensorflow as tf
 
+from deepconsensus.models import data_providers
 from deepconsensus.models import model_configs
 from deepconsensus.models import model_utils
 from deepconsensus.utils import test_utils
@@ -80,36 +81,49 @@ class ModifyParamsTest(parameterized.TestCase):
       self.assertEqual(params.batch_size, params.default_batch_size)
 
 
-class RunInferenceAndWriteResultsTest(absltest.TestCase):
 
-  def test_output_dir_created(self):
-    """Tests that output dir created when it does not exist."""
 
-    out_dir = f'/tmp/output_dir/{uuid.uuid1()}'
-    self.assertFalse(tf.io.gfile.isdir(out_dir))
-    params = model_configs.get_config('transformer_learn_values+test')
-    model_utils.modify_params(params)
-    model = model_utils.get_model(params)
-    checkpoint_path = test_utils.deepconsensus_testdata('model/checkpoint-1')
-    checkpoint = tf.train.Checkpoint(model=model)
-    checkpoint.restore(checkpoint_path)
-    model.compile(
-        optimizer=tf.keras.optimizers.Adam(learning_rate=params.learning_rate),
-        loss=model_utils.get_deepconsensus_loss(params),
-        metrics=model_utils.get_deepconsensus_metrics())
-    model_utils.run_inference_and_write_results(model, out_dir, params)
-    self.assertTrue(tf.io.gfile.isdir(out_dir))
-    inference_output = os.path.join(out_dir, 'inference.csv')
-    self.assertTrue(tf.io.gfile.exists(inference_output))
-    with tf.io.gfile.GFile(inference_output) as inference_output_file:
+class GetStepCountsTest(parameterized.TestCase):
 
-      # Check that model.metric_names was called after real data was fed through
-      # the model, and that metric names are correct.
-      first_line = inference_output_file.readline()
-      self.assertEqual(
-          first_line, 'dataset,loss,' + ','.join([
-              metric.name for metric in model_utils.get_deepconsensus_metrics()
-          ]) + '\n')
+  @parameterized.named_parameters(
+      dict(
+          testcase_name='simple',
+          n_examples_train=1000,
+          n_examples_eval=100,
+          batch_size=10,
+          limit=-1,
+          eval_and_log_every_step=False,
+          expected_step_counts=(100, 10)),
+      dict(
+          testcase_name='with_limit',
+          n_examples_train=1000,
+          n_examples_eval=100,
+          batch_size=10,
+          limit=100,
+          eval_and_log_every_step=False,
+          expected_step_counts=(10, 10)),
+      dict(
+          testcase_name='simple_eval_log_every_step',
+          n_examples_train=1000,
+          n_examples_eval=100,
+          batch_size=10,
+          limit=-1,
+          eval_and_log_every_step=True,
+          expected_step_counts=(1, 1)),
+  )
+  def test_get_step_counts(self, n_examples_train, n_examples_eval, batch_size,
+                           limit, eval_and_log_every_step,
+                           expected_step_counts):
+    params = model_configs.get_config('fc+test')
+    with params.unlocked():
+      params.n_examples_train = n_examples_train
+      params.n_examples_eval = n_examples_eval
+      params.limit = limit
+      params.batch_size = batch_size
+
+    self.assertEqual(
+        model_utils.get_step_counts(params, eval_and_log_every_step),
+        expected_step_counts)
 
 
 if __name__ == '__main__':
