@@ -28,6 +28,7 @@
 """Utility functions being used for data processing."""
 
 import collections
+from collections import abc
 import dataclasses
 import itertools
 from typing import Any, Dict, List, Optional, Union
@@ -38,16 +39,19 @@ import tensorflow as tf
 
 from deepconsensus.models import data_providers
 from deepconsensus.utils import dc_constants
+from deepconsensus.utils import utils
 
 Issue = dc_constants.Issue
 
 
-class SubreadGrouper(collections.Iterator):
+class SubreadGrouper(abc.Iterator):
   """Returns all subreads belonging to a single zmw as a list."""
 
   def __init__(self, subreads_to_ccs, reader_threads):
+    save = pysam.set_verbosity(0)  # Avoid confusing index file warning.
     self.bam_reader = pysam.AlignmentFile(
         subreads_to_ccs, check_sq=False, threads=reader_threads)
+    pysam.set_verbosity(save)
     self.keep_iter = True
     self.subread_group = []
     # Setup subread group.
@@ -102,7 +106,7 @@ def right_pad(arr: np.ndarray, length: int, value: Any) -> np.ndarray:
 
 
 @dataclasses.dataclass
-class Read(collections.Sequence):
+class Read(abc.Sequence):
   """Used to represent ccs alignments."""
   name: str
   bases: np.ndarray
@@ -111,9 +115,11 @@ class Read(collections.Sequence):
   ip: np.ndarray
   sn: np.ndarray
   strand: dc_constants.Strand
-  ccs_idx: np.ndarray = np.empty(0)
+  # base_quality_scores are only used for the ccs read.
+  base_quality_scores: np.ndarray = np.empty(0, dtype=np.uint8)
+  ccs_idx: np.ndarray = np.empty(0, dtype=np.int)
   # truth_idx and truth_range only used with label reads.
-  truth_idx: np.ndarray = np.empty(0)
+  truth_idx: np.ndarray = np.empty(0, np.int)
   # truth range is a dict containing contig, begin, end.
   # It is not modified when slicing is performed.
   # The truth_range['contig'] and truth_idx are used calculate
@@ -122,8 +128,8 @@ class Read(collections.Sequence):
   truth_range: Union[Dict[str, Any], None] = None
 
   # Alignment Variables
-  seq_indices: np.ndarray = np.empty(0)
-  is_insertion: np.ndarray = np.empty(0)
+  seq_indices: np.ndarray = np.empty(0, dtype=np.int)
+  is_insertion: np.ndarray = np.empty(0, dtype=bool)
   seq_len: int = 0
   idx_seq: int = 0
   idx_spaced: int = 0
@@ -157,7 +163,7 @@ class Read(collections.Sequence):
     # When insertions are encountered in the label, add them in
     # to maintain spacing correctly.
     if self.truth_range:
-      while self.is_insertion[self.idx_seq]:
+      while not self.is_out_of_bounds() and self.is_insertion[self.idx_seq]:
         # For label insertions, insert bases.
         self.seq_indices[self.idx_seq] = self.idx_spaced
         self.idx_seq += 1
@@ -193,6 +199,12 @@ class Read(collections.Sequence):
     self.ip = spaced_ip
     self.ccs_idx = spaced_ccs_idx
 
+    # Handle base quality scores. Only present with ccs reads.
+    if self.base_quality_scores.any():
+      spaced_base_quality_scores = np.repeat(-1, seq_len)
+      spaced_base_quality_scores[self.seq_indices] = self.base_quality_scores
+      self.base_quality_scores = spaced_base_quality_scores
+
   @property
   def bases_encoded(self) -> np.ndarray:
     bases_encoded = np.ndarray(
@@ -201,6 +213,11 @@ class Read(collections.Sequence):
       bases_encoded[self.bases == base] = k
     return bases_encoded
 
+  @property
+  def avg_base_quality_score(self) -> float:
+    """Outputs the average CCS base quality score."""
+    return utils.avg_phred(self.base_quality_scores)
+
   @property
   def zmw(self) -> int:
     return int(self.name.split('/')[1])
@@ -257,6 +274,7 @@ class Read(collections.Sequence):
         ip=self.ip[ccs_slice],
         sn=self.sn,
         strand=self.strand,
+        base_quality_scores=self.base_quality_scores[ccs_slice],
         ccs_idx=self.ccs_idx[ccs_slice],
         truth_idx=self.truth_idx[ccs_slice],
         truth_range=self.truth_range)
@@ -270,6 +288,7 @@ class Read(collections.Sequence):
         ip=right_pad(self.ip, pad_width, 0),
         sn=self.sn,
         strand=self.strand,
+        base_quality_scores=right_pad(self.base_quality_scores, pad_width, -1),
         ccs_idx=right_pad(self.ccs_idx, pad_width, -1),
         truth_idx=right_pad(self.truth_idx, pad_width, -1),
         truth_range=self.truth_range)
@@ -278,6 +297,10 @@ class Read(collections.Sequence):
     """Removes gaps from sequence and returns padded."""
     # Useful for reducing label width.
     keep = self.bases != dc_constants.GAP_OR_PAD
+    if self.base_quality_scores.any():
+      base_quality_scores = self.base_quality_scores[keep]
+    else:
+      base_quality_scores = np.empty(0, dtype=np.uint8)
     if sum(keep) > pad_width:
       return None
     return Read(
@@ -288,6 +311,7 @@ class Read(collections.Sequence):
         ip=self.ip[keep],
         sn=self.sn,
         strand=self.strand,
+        base_quality_scores=base_quality_scores,
         ccs_idx=self.ccs_idx[keep],
         truth_idx=self.truth_idx[keep],
         truth_range=self.truth_range).pad(pad_width)
@@ -308,6 +332,7 @@ class Read(collections.Sequence):
         ip=self.ip[r_slice],
         sn=self.sn,
         strand=self.strand,
+        base_quality_scores=self.base_quality_scores[r_slice],
         ccs_idx=self.ccs_idx[r_slice],
         truth_idx=self.truth_idx[r_slice])
 
@@ -378,7 +403,7 @@ class DcConfig:
     return sum(self.feature_rows.values())
 
   @property
-  def tensor_width(self) -> int:
+  def max_length(self) -> int:
     """Returns total rows for tf.Example input."""
     return self.example_width + self.padding
 
@@ -390,7 +415,7 @@ class DcConfig:
         'example_width': str(self.example_width),
         'padding': str(self.padding),
         'tensor_height': str(self.tensor_height),
-        'tensor_width': str(self.tensor_width)
+        'tensor_width': str(self.max_length)
     }
 
 
@@ -553,7 +578,8 @@ class DcExample:
         'subreads': data,
         'subreads/num_passes': self.keep_subreads,
         'name': self.name,
-        'window_pos': self.ccs.ccs_bounds.start
+        'window_pos': self.ccs.ccs_bounds.start,
+        'ccs_base_quality_scores': self.ccs.base_quality_scores
     }
     return features
 
@@ -570,6 +596,8 @@ class DcExample:
     features.feature['name'].bytes_list.value.append(self.name.encode())
     features.feature['window_pos'].int64_list.value.append(
         self.ccs.ccs_bounds.start)
+    features.feature['ccs_base_quality_scores'].int64_list.value.extend(
+        self.ccs.base_quality_scores)
 
     if self.is_training:
       label = self.label.bases_encoded
@@ -644,7 +672,7 @@ def from_features_dict(features_dict: Dict[str, Any],
   ccs = decode_bases(data[ccs_idx])[0]
   sn = data[sn_idx][:, 1]
 
-  ccs_idx = np.repeat(-1, dc_config.tensor_width)
+  ccs_idx = np.repeat(-1, dc_config.max_length)
   ccs_end_pos = ccs_start_pos + dc_config.example_width
   ccs_idx[0:dc_config.example_width] = np.arange(ccs_start_pos, ccs_end_pos)
 
@@ -678,8 +706,12 @@ def from_features_dict(features_dict: Dict[str, Any],
 
 def set_feature(feature, shape):
   """Read in feature and set shape."""
-  feature = np.frombuffer(feature, dtype=dc_constants.NP_DATA_TYPE)
-  feature = feature.reshape(shape)
+  if not tf.executing_eagerly():
+    feature = tf.io.decode_raw(feature, dc_constants.TF_DATA_TYPE)
+    feature = tf.reshape(feature, shape)
+  else:
+    feature = np.frombuffer(feature, dtype=dc_constants.NP_DATA_TYPE)
+    feature = feature.reshape(shape)
   return feature
 
 
@@ -689,14 +721,24 @@ def tf_example_to_features_dict(tf_example_proto_str, inference=False):
       tf_example_proto_str, inference=inference)
 
   for key, val in features.items():
-    features[key] = val.numpy()
+    if tf.executing_eagerly():
+      features[key] = val.numpy()
 
   # Cast types
-  features['name'] = str(features['name'][0], 'UTF-8')
+  if 'name' in features.keys():
+    if tf.executing_eagerly():
+      features['name'] = str(features['name'][0], 'UTF-8')
+    else:
+      features['name'] = features['name'][0]
   features['subreads/num_passes'] = int(features['subreads/num_passes'])
 
   features['subreads'] = set_feature(features['subreads/encoded'],
                                      features['subreads/shape'])
+  dc_config = DcConfig.from_shape(features['subreads/shape'])
+  features['subreads'] = data_providers.format_rows(
+      features['subreads'],
+      max_passes=dc_config.max_passes,
+      max_length=dc_config.max_length)
   del features['subreads/encoded']
   if not inference:
     features['label'] = set_feature(features['label/encoded'],
@@ -705,20 +747,19 @@ def tf_example_to_features_dict(tf_example_proto_str, inference=False):
   return features
 
 
-def fetch_ccs_seq(ccs_seqname: str,
-                  ccs_fasta: pysam.libcfaidx.FastaFile) -> Read:
-  """Fetches a ccs sequence by name."""
-  ccs_seq = ccs_fasta.fetch(ccs_seqname)
-  ccs_seq = np.array(ccs_seq, 'c')
-  # The ccs ref sequences are 1:len(ccs_seq).
+def construct_ccs_read(
+    ccs_bam_read: pysam.libcalignedsegment.AlignedSegment) -> Read:
+  """Constructs a Read with quality scores using a ccs bam read."""
+  ccs_seq = np.array(ccs_bam_read.seq, 'c')
   return Read(
-      name=ccs_seqname,
+      name=ccs_bam_read.qname,
       bases=ccs_seq,
       cigar=np.repeat(np.uint8(pysam.CMATCH), len(ccs_seq)),
       pw=np.repeat(np.uint8(0), len(ccs_seq)),
       ip=np.repeat(np.uint8(0), len(ccs_seq)),
       sn=np.repeat(0, 4),
       strand=dc_constants.Strand.UNKNOWN,
+      base_quality_scores=np.array(ccs_bam_read.query_qualities),
       ccs_idx=np.arange(len(ccs_seq)))
 
 
@@ -816,7 +857,7 @@ def expand_clip_indent(read: pysam.AlignedSegment,
     new_ip[read_idx >= 0] = ip_vals
     sn = np.array(read.get_tag('sn'))
   else:
-    sn = np.empty(0)
+    sn = np.empty(0, dtype=np.uint8)
 
   # Extract additional read properties.
   cigar_seq = itertools.chain.from_iterable([[x] * y for x, y in read.cigar])
@@ -891,7 +932,8 @@ def space_out_subreads(subreads: List[Read]) -> List[Read]:
         r.add_gap()
       else:
         # In all other cases, just take the next base and move on.
-        r.move()
+        if not r.is_out_of_bounds():
+          r.move()
         if r.is_out_of_bounds():
           # Finally, format reads with spacing.
           r.done = True
@@ -905,7 +947,7 @@ def space_out_subreads(subreads: List[Read]) -> List[Read]:
 
 
 def create_proc_feeder(subreads_to_ccs: str,
-                       ccs_fasta: str,
+                       ccs_bam: str,
                        dc_config: DcConfig,
                        truth_bed: Optional[str] = None,
                        truth_to_ccs: Optional[str] = None,
@@ -917,7 +959,10 @@ def create_proc_feeder(subreads_to_ccs: str,
 
   # Initiate files
   subread_grouper = SubreadGrouper(subreads_to_ccs, bam_reader_threads)
-  ccs_fasta = pysam.FastaFile(ccs_fasta)
+
+  save = pysam.set_verbosity(0)  # Avoid confusing index file warning.
+  ccs_bam_h = pysam.AlignmentFile(ccs_bam, check_sq=False)
+  pysam.set_verbosity(save)
 
   is_training = truth_bed and truth_to_ccs and truth_split
   if is_training:
@@ -932,8 +977,17 @@ def create_proc_feeder(subreads_to_ccs: str,
       subreads = list(map(expand_clip_indent, read_set))
       ccs_seqname = '/'.join(subreads[0].name.split('/')[:2] + ['ccs'])
       # Fetch ccs sequence and append to subread set.
-      ccs_seq = fetch_ccs_seq(ccs_seqname, ccs_fasta)
-      subreads.append(ccs_seq)
+      while True:
+        ccs_bam_read = next(ccs_bam_h)
+        if ccs_bam_read.qname == ccs_seqname:
+          break
+      # If ccs read is not present in subread_to_ccs bam, throw error.
+      if ccs_bam_read.qname != ccs_seqname:
+        raise ValueError(f'ccs bam does not contain {ccs_seqname}')
+
+      ccs_read = construct_ccs_read(ccs_bam_read)
+      subreads.append(ccs_read)
+
       if is_training:
         # Fetch truth to ccs alignment.
         truth_range = truth_ref_coords.get(ccs_seqname, None)
